import { linter } from '@codemirror/lint';
import { parserWrapper, validateSyntax } from '@neo4j-cypher/language-support';
import { DiagnosticSeverity } from 'vscode-languageserver-types';
import workerpool from 'workerpool';
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore ignore: https://v3.vitejs.dev/guide/features.html#import-with-query-suffixes
import WorkerURL from './lintWorker?url&worker';
const pool = workerpool.pool(WorkerURL, {
    minWorkers: 2,
    workerOpts: { type: 'module' },
    workerTerminateTimeout: 2000,
});
let lastSemanticJob;
export const cypherLinter = (config) => linter((view) => {
    if (!config.lint) {
        return [];
    }
    const query = view.state.doc.toString();
    const syntaxErrors = validateSyntax(query, config.schema ?? {});
    return syntaxErrors.map((diagnostic) => ({
        from: diagnostic.offsets.start,
        to: diagnostic.offsets.end,
        severity: diagnostic.severity === DiagnosticSeverity.Error
            ? 'error'
            : 'warning',
        message: diagnostic.message,
    }));
});
export const semanticAnalysisLinter = (config) => linter(async (view) => {
    if (!config.lint) {
        return [];
    }
    const query = view.state.doc.toString();
    if (query.length === 0) {
        return [];
    }
    // we want to avoid the ANTLR4 reparse in the worker thread, this should hit our main thread cache
    const parse = parserWrapper.parse(query);
    const statements = parse.statementsParsing;
    const anySyntacticError = statements.filter((statement) => statement.diagnostics.length !== 0)
        .length > 0;
    if (anySyntacticError) {
        return [];
    }
    try {
        if (lastSemanticJob !== undefined && !lastSemanticJob.resolved) {
            void lastSemanticJob.cancel();
        }
        const proxyWorker = (await pool.proxy());
        lastSemanticJob = proxyWorker.validateSemantics(query, config.schema ?? {});
        const result = await lastSemanticJob;
        return result.map((diag) => {
            return {
                from: diag.offsets.start,
                to: diag.offsets.end,
                severity: diag.severity === DiagnosticSeverity.Error ? 'error' : 'warning',
                message: diag.message,
            };
        });
    }
    catch (err) {
        if (!(err instanceof workerpool.Promise.CancellationError)) {
            console.error(String(err) + ' ' + query);
        }
    }
    return [];
});
export const cleanupWorkers = () => {
    void pool.terminate();
};
//# sourceMappingURL=syntaxValidation.js.map